#include <linux/kernel.h>
#include "cxt_mgr.h"
#include "framegrabber.h"
#include "mem_model.h"
#include "v4l2_model.h"
#include "board_v4l2.h"
#include "debug.h"
#include "pci_model.h"
#include "i2c_model.h"
#include "task_model.h"
#include "gpio_model.h"
#include "aver_xilinx.h"
#include "gv7601.h"
#include "adv7604.h"
#include "m24c02.h"
#include "board_alsa.h"
#include "board_gpio.h"
#include "pca9539a.h"

enum cl311mn_input_e {
	CL311MN_HDMI_INPUT = 0,
	CL311MN_VGA_INPUT,
	CL311MN_SDI_INPUT,
	
	CL311MN_INPUT_COUNT,
};

enum cl311mn_i2c_chip_e {
	CL311MN_I2C_CHIP_M24C02,
	CL311MN_I2C_CHIP_GV7601,
	CL311MN_I2C_CHIP_ADV7604, //need to be checked 0117
	CL311MN_I2C_CHIP_PCA9539A,
	CL311MN_I2C_CHIP_COUNT,
};

typedef struct {
	BASIC_CXT_HANDLE_DECLARE;
	framegrabber_handle_t fg_handle;
	v4l2_model_handle_t v4l2_handle;
	task_model_handle_t task_model_handle;
	handle_t aver_xilinx_handle;
	handle_t i2c_chip_handle[CL311MN_I2C_CHIP_COUNT];
	task_handle_t check_signal_task_handle;
	adv7604_frameinfo_t adv7604_cur_frameinfo;
	gv7601_frameinfo_t gv7601_cur_frameinfo;
} board_v4l2_context_t;

typedef struct {
	const char *name;
	int index;
} board_chip_desc_t;

static board_chip_desc_t cl311mn_chip_desc[CL311MN_I2C_CHIP_COUNT] = {
	[CL311MN_I2C_CHIP_M24C02] = {
		.name = M24C02_DRVNAME,
		.index = 1,
	},
	[CL311MN_I2C_CHIP_GV7601] = {
		.name = GV7601_DRVNAME,
		.index = 1,
	},
	
	[CL311MN_I2C_CHIP_ADV7604] = {
		.name = ADV7604_DRVNAME,
		.index = 1,
	},
	[CL311MN_I2C_CHIP_PCA9539A] = {
		.name = PCA9539A_DRVNAME,
		.index = 1,
	},
};

static framegrabber_setup_input_info_t cl311mn_input_info[] = {
	[CL311MN_HDMI_INPUT] = {
		.name = "HDMI",
		.support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	[CL311MN_VGA_INPUT] = {
		.name = "Component/VGA",
		.support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT | REFRESHRATE_85_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT | REFRESHRATE_75_BIT,
			[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
			[FRAMESIZE_1920x1200] = REFRESHRATE_60_BIT,
		},
	},
	[CL311MN_SDI_INPUT] = {
		.name = "SDI",
		.support_framesize_info = {
			[FRAMESIZE_640x480]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_702x576]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x400]   = REFRESHRATE_60_BIT,
			[FRAMESIZE_720x480]   = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT,
			[FRAMESIZE_720x576]   = REFRESHRATE_50_BIT,
			[FRAMESIZE_800x600]   = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_72_BIT | REFRESHRATE_60_BIT | REFRESHRATE_56_BIT,
			[FRAMESIZE_1024x768]  = REFRESHRATE_85_BIT | REFRESHRATE_75_BIT | REFRESHRATE_70_BIT | REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x864]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1152x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x720]  = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT,
			[FRAMESIZE_1280x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x800]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x960]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1280x1024] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1360x768]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1400x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x480]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x576]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1440x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x900]  = REFRESHRATE_60_BIT,
			[FRAMESIZE_1600x1200] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1680x1050] = REFRESHRATE_60_BIT,
			[FRAMESIZE_1920x1080] = REFRESHRATE_60_BIT | REFRESHRATE_59_94_BIT | REFRESHRATE_50_BIT | REFRESHRATE_30_BIT | REFRESHRATE_29_97_BIT | \
									REFRESHRATE_25_BIT | REFRESHRATE_24_BIT | REFRESHRATE_23_98_BIT,
		},
	},
	
	[CL311MN_INPUT_COUNT] = {
		.name = NULL,
	},
};


static framegrabber_property_t cl311mn_property = {
	.name = "CL311MN",
	.input_setup_info = cl311mn_input_info,
	.support_out_pixfmt_mask = FRAMEGRABBER_PIXFMT_YUYV_BIT,
	//.max_supported_line_width = 3840,
	.max_frame_size = 4096 * 2160,
};

static v4l2_model_device_setup_t device_info =
{
	.type = DEVICE_TYPE_GRABBER,
	.driver_name = "CL311MN",
	.card_name = "AVerMedia CL311MN",
	.capabilities = V4L2_MODEL_CAPS_CAPTURE_BIT | V4L2_MODEL_CAPS_READWRITE_BIT | V4L2_MODEL_CAPS_STREAMING_BIT,
	.buffer_type = V4L2_MODEL_BUF_TYPE_DMA_SG,
};

static void *board_v4l2_alloc(void);
static void board_v4l2_release(void *cxt);

static void fg_s_input(framegrabber_handle_t handle, int input);
static void fg_stream_on(framegrabber_handle_t handle);
static void fg_stream_off(framegrabber_handle_t handle);
static void fg_wr_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid);
static void fg_rd_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid);

static void cl311mn_adv7604_event(void *cxt, adv7604_event_e event);
static void cl311mn_gv7601_event(void *cxt, gv7601_event_e event);

static void video_buffer_done(void *data);

static framegrabber_interface_t cl311mn_ops = {
	.s_input      = fg_s_input,
	.stream_on    = fg_stream_on,
	.stream_off   = fg_stream_off,
	.wr_edid      = fg_wr_edid,
	.rd_edid      = fg_rd_edid,
};

static void *board_v4l2_alloc()
{
	board_v4l2_context_t *cxt;

	cxt = mem_model_alloc_buffer(sizeof(*cxt));

	return cxt;
}

static void board_v4l2_release(void *cxt)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;

	if (board_v4l2_cxt) {
		if (board_v4l2_cxt->check_signal_task_handle) {
			task_model_release_task(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle);
			board_v4l2_cxt->check_signal_task_handle = NULL;
		}
		cxt_manager_unref_context(board_v4l2_cxt->task_model_handle);
		cxt_manager_unref_context(board_v4l2_cxt->aver_xilinx_handle);
		mem_model_free_buffer(board_v4l2_cxt);
	}
}

static void v4l2_stream_on(v4l2_model_callback_parameter_t *cb_info)
{
	board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	debug_msg("%s %p\n", __func__, board_v4l2_cxt);
}

static void v4l2_stream_off(v4l2_model_callback_parameter_t *cb_info)
{
	board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	debug_msg("%s %p\n", __func__, board_v4l2_cxt);
}

static void v4l2_buffer_prepare(v4l2_model_callback_parameter_t *cb_info)
{
	board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	v4l2_model_buffer_info_t *buffer_info = cb_info->u.buffer_prepare_info.buffer_info;

	//debug_msg("%s %p buffer_info %p\n", __func__, board_v4l2_cxt, buffer_info);
	if (buffer_info) {
		int i;
		v4l2_model_buf_desc_t *desc;
		unsigned framebufsize, remain;
		int width, height;
		unsigned bytesperline;

		framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle, &width, &height);
		bytesperline = framegrabber_g_out_bytesperline(board_v4l2_cxt->fg_handle);
		framebufsize = bytesperline * height;
		//debug_msg("%s %dx%d framesize %u\n", __func__, width, height, framebufsize);
		//debug_msg("buf type %d count %d\n", buffer_info->buf_type, buffer_info->buf_count[0]);
		for (i = 0, desc = buffer_info->buf_info[0], remain = framebufsize; i < buffer_info->buf_count[0]; i++) {
			//debug_msg("addr %08x size %x\n", desc[i].addr, desc[i].size);
			if (remain >= desc[i].size) {
				aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle, desc[i].addr, desc[i].size, 0);
				remain -= desc[i].size;
			} else {
				aver_xilinx_add_to_cur_desclist(board_v4l2_cxt->aver_xilinx_handle, desc[i].addr, remain, 0);
				remain = 0;
				break;
			}
		}

		aver_xilinx_active_current_desclist(board_v4l2_cxt->aver_xilinx_handle, video_buffer_done, board_v4l2_cxt, 0);
	}
}

static void v4l2_buffer_init(v4l2_model_callback_parameter_t *cb_info)
{
	//board_v4l2_context_t *board_v4l2_cxt = cb_info->asso_data;
	v4l2_model_buffer_info_t *buffer_info = cb_info->u.buffer_prepare_info.buffer_info;

	//mesg("%s %p buffer_info %p\n", __func__, board_v4l2_cxt, buffer_info);
	if (buffer_info) {
		int i;
		v4l2_model_buf_desc_t *desc;
		//mesg("buf type %d count %d\n", buffer_info->buf_type, buffer_info->buf_count[0]);
		for (i = 0, desc = buffer_info->buf_info[0]; i < buffer_info->buf_count[0]; i++) {
			//mesg("addr %08x size %x\n", desc[i].addr,desc[i].size);
		}
	}
}

static void fg_s_input(framegrabber_handle_t handle, int input)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	//cxt_mgr_handle_t cxt_mgr = get_cxt_manager_from_context(board_v4l2_cxt);
	
	handle_t adv7604_handle = board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_ADV7604];
	handle_t gv7601_handle = board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_GV7601];
	
	printk("%s input=%d\n", __func__, input);
	#if 0
	adv7604_enable_dark_screen(adv7604_handle, AVER_TRUE);
	#endif
	framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
	//input =2;
	//aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,input);
#if 1
	if (input == CL311MN_HDMI_INPUT /*|| input == CL311MN_VGA_INPUT*/) {
		//board_set_gpio(cxt_mgr, BOARD_GPIO_ADV7604_I2S_EN, GPIOIC_VALUE_HIGH);
		
		//board_set_gpio(cxt_mgr, BOARD_GPIO_AIC3101_I2S_EN, GPIOIC_VALUE_HIGH);

		adv7604_set_input(adv7604_handle, ADV7604_MODE_HDMI);
		
	    aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,input);
	    
		gv7601_unregister_callback(gv7601_handle);
        //need to add 0117
		adv7604_register_callback(adv7604_handle, cl311mn_adv7604_event, board_v4l2_cxt);
        printk("%s CL311MN_HDMI_INPUT\n", __func__);
	} 
	else if (input == CL311MN_VGA_INPUT) {
		//adv7604_set_sync_ch(adv7604_handle, ADV7604_SYNC_CH2);
		adv7604_set_input(adv7604_handle, ADV7604_MODE_GR);
		aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,input);
		 
		gv7601_unregister_callback(gv7601_handle);
        //need to add 0117
		adv7604_register_callback(adv7604_handle, cl311mn_adv7604_event, board_v4l2_cxt);
        printk("%s CL311MN_VGA_INPUT\n", __func__);
	}
	
	else if (input == CL311MN_SDI_INPUT) {
		
		//board_set_gpio(cxt_mgr, BOARD_GPIO_AIC3101_I2S_EN, GPIOIC_VALUE_HIGH);
		
        aver_xilinx_vip_select_input(board_v4l2_cxt->aver_xilinx_handle,input);
        
		adv7604_unregister_callback(adv7604_handle);
        //need to add 0117
		gv7601_register_callback(gv7601_handle, cl311mn_gv7601_event, board_v4l2_cxt);
		printk("%s CL311MN_SDI_INPUT\n", __func__);
	}
#endif	
}

static void fg_stream_on(framegrabber_handle_t handle)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	//cxt_mgr_handle_t cxt_mgr = get_cxt_manager_from_context(board_v4l2_cxt);
	//handle_t cdce913_handle = board_v4l2_cxt->i2c_chip_handle[CL311M1_I2C_CHIP_CDCE913];
	handle_t adv7604_handle=board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_ADV7604]; 
	aver_xilinx_video_process_cfg_t vip_cfg;
	aver_xilinx_audio_process_cfg_t aup_cfg;
	int width, height, out_width, out_height, out_framerate;
	int crop_left, crop_top, crop_width, crop_height;
	framegrabber_framemode_e framemode;
	int input;
	int audio_rate=0;
	enum adv7604_audio_sample fe_audioinfo=0;
	const framegrabber_pixfmt_t *pixfmt=framegrabber_g_out_pixelfmt(handle);
	
	printk(">>%s \n", __func__);

	input = framegrabber_g_input(board_v4l2_cxt->fg_handle);

    printk("%s input=%d\n", __func__, input);

	if (input == CL311MN_HDMI_INPUT) 
	{
		adv7604_get_audioinfo(adv7604_handle,&fe_audioinfo);
	}
	else if(input == CL311MN_SDI_INPUT)
	{
		fe_audioinfo = ADV7604_AUDIO_RATE_48000;
	}
	
	audio_rate = fe_audioinfo;
	//framegrabber_s_input_audioinfo(board_v4l2_cxt->fg_handle,audio_rate);
	
	switch (audio_rate)
    {
		case ADV7604_AUDIO_RATE_32000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_32000;
		break;
		case ADV7604_AUDIO_RATE_44100:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_44100;
		break;
		case ADV7604_AUDIO_RATE_48000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		break;
		case ADV7604_AUDIO_RATE_88200:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_88200;
		break;
		case ADV7604_AUDIO_RATE_96000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_96000;
		break;
		case ADV7604_AUDIO_RATE_176400:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_176400;
		break;
		case ADV7604_AUDIO_RATE_192000:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_192000;
		break;
		default:
		vip_cfg.audio_rate=AVER_XILINX_AUDIO_48000;
		break;
	}
    printk("%s...vip_cfg.audio_rate_index=%d\n",__func__,vip_cfg.audio_rate);
		
	framemode = framegrabber_g_input_framemode(board_v4l2_cxt->fg_handle);
	framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle, &width, &height);
	framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle, &out_width, &out_height);
	out_framerate = framegrabber_g_out_framerate(board_v4l2_cxt->fg_handle);
	framegrabber_g_crop_rect(board_v4l2_cxt->fg_handle, &crop_left, &crop_top, &crop_width, &crop_height);

	vip_cfg.is_interlace = (FRAMEGRABBER_FRAMEMODE_INTERLACED == framemode) ? AVER_TRUE : AVER_FALSE;
	vip_cfg.in_framesize.width = width;
	vip_cfg.in_framesize.height = height;
	vip_cfg.out_framesize.width = out_width;
	vip_cfg.out_framesize.height = out_height;
	vip_cfg.out_framerate = out_framerate;
	//vip_cfg.clip_start.x = crop_left;
	//vip_cfg.clip_start.y = crop_top;
	//vip_cfg.clip_size.width = crop_width;
	//vip_cfg.clip_size.height = crop_height;
	vip_cfg.clip_start.x = 0;
	vip_cfg.clip_start.y = 0;
	vip_cfg.clip_size.width = width;
	vip_cfg.clip_size.height = height;
	printk(">>>>>>>>>>>>>>>>%s in %dx%d out %dx%d %d\n", __func__, width, height, out_width, out_height, out_framerate);

	//vip_cfg.valid_mask = FRAMERATE_CFG_VALID_MASK | CLIP_CFG_VALID_MASK | SCALER_CFG_VALID_MASK;
	//vip_cfg.valid_mask = FRAMERATE_CFG_VALID_MASK | SCALER_CFG_VALID_MASK;
	//vip_cfg.pixel_clock = 0;
	
    vip_cfg.valid_mask=SCALER_CFG_VALID_MASK|FRAMERATE_CFG_VALID_MASK;
    
    vip_cfg.in_framerate = framegrabber_g_input_framerate(board_v4l2_cxt->fg_handle);

	if((vip_cfg.in_framesize.width == 720) && (vip_cfg.in_framesize.height == 244))
	{
    	vip_cfg.is_interlace = AVER_FALSE;
    }
    
    if (vip_cfg.is_interlace==AVER_TRUE)
        vip_cfg.in_framerate = vip_cfg.in_framerate *2; //same as fpga detect
    else
        vip_cfg.in_framerate = vip_cfg.in_framerate;
	
#if 1 //audio path
	if (input == CL311MN_HDMI_INPUT) {
		aup_cfg.input = AVER_XILINX_INPUT_0; 
		vip_cfg.input_channel = AVER_XILINX_INPUT_0;
	} else if (input == CL311MN_VGA_INPUT) {
		aup_cfg.input = AVER_XILINX_INPUT_1; 
		vip_cfg.input_channel = AVER_XILINX_INPUT_1; 
	} else
	{
		aup_cfg.input = AVER_XILINX_INPUT_2; 
		vip_cfg.input_channel = AVER_XILINX_INPUT_2; 
	}
#endif

	aver_xilinx_config_audio_process(board_v4l2_cxt->aver_xilinx_handle, &aup_cfg);

	vip_cfg.pixel_format_out = pixfmt->pixfmt_out;
	aver_xilinx_config_video_process(board_v4l2_cxt->aver_xilinx_handle, &vip_cfg, 0);
	aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_TRUE, 0);

	//task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 100000);
}

static void fg_stream_off(framegrabber_handle_t handle)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
        printk(">>%s \n", __func__);	
	aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_FALSE, 0);
}

static void fg_wr_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	handle_t m24c02_handle = board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_M24C02];

	m24c02_wr_edid(m24c02_handle, start_block, blocks, edid);
}

static void fg_rd_edid(framegrabber_handle_t handle, int start_block, int blocks, U8_T *edid)
{
	board_v4l2_context_t *board_v4l2_cxt = framegrabber_get_data(handle);
	handle_t m24c02_handle = board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_M24C02];

	m24c02_rd_edid(m24c02_handle, start_block, blocks, edid);
}

static void video_buffer_done(void *data)
{
	board_v4l2_context_t *board_v4l2_cxt = data;

	//mesg("%s board_v4l2_cxt %p\n", __func__, board_v4l2_cxt);
	if (board_v4l2_cxt) {
		v4l2_model_buffer_done(board_v4l2_cxt->v4l2_handle);
	}
}

static void check_signal_stable_task(void *data)
{
	board_v4l2_context_t *board_v4l2_cxt = data;
	aver_xilinx_frame_info_t detected_frameinfo;
	
	//framegrabber_framemode_e framemode;
	int width, height, stable =0;
	BOOL_T is_interlace;

	framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle, &width, &height);
	aver_xilinx_get_frameinfo(board_v4l2_cxt->aver_xilinx_handle, &detected_frameinfo, &stable, 0);
	is_interlace = framegrabber_g_input_interlace(board_v4l2_cxt->fg_handle);
	if (detected_frameinfo.width != width || detected_frameinfo.height != height)
	{
		debug_msg("%s detected size %dx%d != fe size %dx%d\n", __func__, detected_frameinfo.width, detected_frameinfo.height,
			width, height);

		task_model_run_task_after(board_v4l2_cxt->task_model_handle, 
			board_v4l2_cxt->check_signal_task_handle, 100000);
		return;
	}

	if(width==detected_frameinfo.width && height==detected_frameinfo.height && (is_interlace==detected_frameinfo.is_interlace))
    {
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT,FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
    }
    else
    {
        //framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0); 
        framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);  
        framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0); 
        //printk("set mask as LOCKED_BIT=0\n");
    }  

	//aver_xilinx_enable_video_streaming(board_v4l2_cxt->aver_xilinx_handle, AVER_TRUE, 0);
#if 0
	debug_msg("%s detected size %dx%d\n", __func__, detected_frameinfo.width, detected_frameinfo.height);
	framemode = framegrabber_g_input_framemode(board_v4l2_cxt->fg_handle);
	is_interlace = (framemode == FRAMEGRABBER_FRAMEMODE_INTERLACED) ? AVER_TRUE : AVER_FALSE;
	framegrabber_g_input_framesize(board_v4l2_cxt->fg_handle, &width, &height);
	if (width == detected_frameinfo.width &&
		height == detected_frameinfo.height &&
		is_interlace == detected_frameinfo.is_interlace)
	{
		framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,
			FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
	}
	else 
	{
		debug_msg("%s adjust in %dx%d to %dx%d\n", __func__, width, height, detected_frameinfo.width, detected_frameinfo.height);
		framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle, detected_frameinfo.width, detected_frameinfo.height);
		framegrabber_g_out_framesize(board_v4l2_cxt->fg_handle, &width, &height);
		if (width == 0 && height == 0)
			framegrabber_s_out_framesize(board_v4l2_cxt->fg_handle, detected_frameinfo.width, detected_frameinfo.height);
		framegrabber_s_crop_rect(board_v4l2_cxt->fg_handle, 0, 0, detected_frameinfo.width, detected_frameinfo.height);
		if (is_interlace != detected_frameinfo.is_interlace)
		{
			framemode = (detected_frameinfo.is_interlace) ? FRAMEGRABBER_FRAMEMODE_INTERLACED : FRAMEGRABBER_FRAMEMODE_PROGRESS;
			framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle, framemode);
		}

		task_model_run_task_after(board_v4l2_cxt->task_model_handle, 
			board_v4l2_cxt->check_signal_task_handle, 100); // run again to comfirm
	}
#endif
}
 //need to add 0117
static void cl311mn_adv7604_event(void *cxt, adv7604_event_e event)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;
	//handle_t aver_xilinx_handle = board_v4l2_cxt->aver_xilinx_handle;
	handle_t adv7604_handle = board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_ADV7604];
	
	adv7604_frameinfo_t *fe_frameinfo=&board_v4l2_cxt->adv7604_cur_frameinfo; 
	//adv7604_frameinfo_t *fe_frameinfo; 
    //enum adv7619_audio_sample *fe_audioinfo=&board_v4l2_cxt->cur_fe_audioinfo; 
    //enum adv7604_audio_sample *fe_audioinfo; 

	switch(event) {
		case ADV7604_YPBPR_LOCK:
		case ADV7604_HDMI_LOCK:
		case ADV7604_LOCK:
		///case ADV7604_NOTIFY_LOCKED:
		///case ADV7604_NOTIFY_SYNCED: //need to check 0125
		    //adv7604_enable_streaming(adv7604_handle,AVER_TRUE);
            //adv7604_enable_streaming(adv7604_handle,AVER_TRUE);
			//adv7604_get_frameinfo(adv7604_handle, &frameinfo);	
			adv7604_get_frameinfo(adv7604_handle,fe_frameinfo);
			 
            framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,fe_frameinfo->framerate, 0);
            
            framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,fe_frameinfo->is_interlace);
            framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle, fe_frameinfo->is_interlace);
            
            framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,fe_frameinfo->width,fe_frameinfo->height); //tt 0602
			{
				#if 0
				mdin400_enable_dark_screen(mdin400_handle, AVER_FALSE); 
				#endif
				framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_OK_BIT);
				/*framegrabber_mask_s_status(board_v4l2_cxt->fg_handle,
					FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);*/
			}
			task_model_run_task_after(board_v4l2_cxt->task_model_handle, board_v4l2_cxt->check_signal_task_handle, 50000);
			//framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle, fe_frameinfo->width, fe_frameinfo->height);
			//printk("ADV7604_LOCK_OUT\n");
			break;

		case ADV7604_UNLOCK:
		case ADV7604_HDMI_UNLOCK:
		case ADV7604_YPBPR_UNLOCK:
		///case ADV7604_NOTIFY_NOSIGNAL:
		///case ADV7604_NOTIFY_NOSYNC:
		    //adv7604_enable_streaming(adv7604_handle,AVER_FALSE);
			//printk("ADV7604_UNLOCK\n");
			
			#if 0
			mdin400_enable_dark_screen(mdin400_handle, AVER_TRUE);
			#endif
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			//framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0);
			framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
			break;

		case ADV7604_HDCP:
		        printk("ADV7604_HDCP\n");
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SYNC_BIT);
			break;
			
		case ADV7604_HOTPLUG:
			printk("aver_xilinx_hdmi_hotplug run!\n");
			aver_xilinx_hdmi_hotplug(board_v4l2_cxt->aver_xilinx_handle);
			break;
		
		default:
			break;
	}
}

// to be checked 0205
static void cl311mn_gv7601_event(void *cxt, gv7601_event_e event)
{
	board_v4l2_context_t *board_v4l2_cxt = cxt;
	//handle_t aver_xilinx_handle = board_v4l2_cxt->aver_xilinx_handle;
	handle_t gv7601_handle = board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_GV7601];
	gv7601_frameinfo_t *fe_frameinfo=&board_v4l2_cxt->gv7601_cur_frameinfo; 
	//gv7601_frameinfo_t *fe_frameinfo; 
	//adv7601_system_type_e system_type; 
	//GV7601_eResolutionType_t system_type; //tbd 0415 for HD/SD ?

	debug_msg("%s event %d\n", __func__, event);
	switch(event) {
		case GV7601_LOCK:
			debug_msg("GV7601_LOCK\n");
			printk("GV7601_LOCK\n");
			//gv7601_get_frameinfo(gv7601_handle, &frameinfo);
			gv7601_get_frameinfo(gv7601_handle,fe_frameinfo);
			
			framegrabber_s_input_framerate(board_v4l2_cxt->fg_handle,fe_frameinfo->framerate, 0);
            framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,fe_frameinfo->width,fe_frameinfo->height);      
            framegrabber_s_input_interlace(board_v4l2_cxt->fg_handle,fe_frameinfo->is_interlace);
            
            printk("%s wxh=%dx%d; framerate=%d; is interlace=%d\n",__func__,(int)fe_frameinfo->width,
            (int)fe_frameinfo->height,(int)fe_frameinfo->framerate,fe_frameinfo->is_interlace);	
            
            framegrabber_s_input_framemode(board_v4l2_cxt->fg_handle, fe_frameinfo->is_interlace);
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_OK);
			framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT);
			break;

		case GV7601_UNLOCK:
			debug_msg("GV7601_UNLOCK\n");
			printk("GV7601_UNLOCK\n");

			//gv7601_enable_dark_screen(mdin400_handle, AVER_TRUE);
			framegrabber_s_input_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_INPUT_STATUS_NO_SIGNAL_BIT);
			//framegrabber_mask_s_status(board_v4l2_cxt->fg_handle, FRAMEGRABBER_STATUS_SIGNAL_LOCKED_BIT, 0);
			framegrabber_s_input_framesize(board_v4l2_cxt->fg_handle,0,0);
			break;

		case ADV7604_HOTPLUG:
			aver_xilinx_hdmi_hotplug(board_v4l2_cxt->aver_xilinx_handle);
			break;
		
		default:
			break;
	}
}

void board_v4l2_init(cxt_mgr_handle_t cxt_mgr)
{
	framegrabber_handle_t framegrabber_handle;
	board_v4l2_context_t *board_v4l2_cxt = NULL;
	v4l2_model_handle_t v4l2_handle;
	task_model_handle_t task_model_handle = NULL;
	handle_t i2c_mgr = NULL;
	handle_t aver_xilinx_handle = NULL;
	task_handle_t task_handle = NULL;
	int i;
	enum {
		BOARD_V4L2_OK = 0,
		BOARD_V4L2_ERROR_CXT_MGR,
		BOARD_V4L2_ERROR_FRAMEGRABBER_INIT,
		BOARD_V4L2_ERROR_V4L2_MODEL_INIT,
		BOARD_V4L2_ERROR_ALLOC_CXT,
		BOARD_V4L2_ERROR_GET_MAIN_CHIP_HANDLE,
		BOARD_V4L2_ERROR_GET_I2C_MGR,
		BOARD_V4L2_ERROR_GET_TASK_MGR,
		BOARD_V4L2_ERROR_ALLOC_TASK,
	} err = BOARD_V4L2_OK;

	do {
		if (!cxt_mgr) {
			err = BOARD_V4L2_ERROR_CXT_MGR;
			break;
		}

		framegrabber_handle = framegrabber_init(cxt_mgr, &cl311mn_property, &cl311mn_ops);
		if (framegrabber_handle == NULL) {
			err = BOARD_V4L2_ERROR_FRAMEGRABBER_INIT;
			break;
		}

		board_v4l2_cxt = cxt_manager_add_cxt(cxt_mgr, BOARD_V4L2_CXT_ID, board_v4l2_alloc, board_v4l2_release);
		if (!board_v4l2_cxt) {
			err = BOARD_V4L2_ERROR_ALLOC_CXT;
			break;
		}

		board_v4l2_cxt->fg_handle = framegrabber_handle;
		framegrabber_set_data(framegrabber_handle, board_v4l2_cxt);
		
		v4l2_handle = v4l2_model_init(cxt_mgr, &device_info, framegrabber_handle, 0);
		if (v4l2_handle == NULL) {
			err = BOARD_V4L2_ERROR_V4L2_MODEL_INIT;
			break;
		}
		board_v4l2_cxt->v4l2_handle = v4l2_handle;

		aver_xilinx_handle = cxt_manager_get_context(cxt_mgr, AVER_XILINX_CXT_ID, 0);
		if (aver_xilinx_handle == NULL) {
			err = BOARD_V4L2_ERROR_GET_MAIN_CHIP_HANDLE;
			break;
		}
		board_v4l2_cxt->aver_xilinx_handle = aver_xilinx_handle;

		i2c_mgr = cxt_manager_get_context(cxt_mgr, I2C_CXT_ID, 0);
		if (i2c_mgr == NULL) {
			err = BOARD_V4L2_ERROR_GET_I2C_MGR;
			break;
		}

		for (i = 0; i < CL311MN_I2C_CHIP_COUNT; i++) {
			board_v4l2_cxt->i2c_chip_handle[i] = i2c_model_get_nth_driver_handle(i2c_mgr, 
				 cl311mn_chip_desc[i].name, cl311mn_chip_desc[i].index);

			printk("i2c_chip_handle[%d]=%p\n", i, board_v4l2_cxt->i2c_chip_handle[i]);
		}

		task_model_handle = cxt_manager_get_context(cxt_mgr, TASK_MODEL_CXT_ID, 0);
		if (task_model_handle == NULL) {
			err = BOARD_V4L2_ERROR_GET_TASK_MGR;
			break;
		}
		board_v4l2_cxt->task_model_handle = task_model_handle;
		cxt_manager_ref_context(task_model_handle);

		task_handle = task_model_create_task(task_model_handle, check_signal_stable_task, board_v4l2_cxt, "check_signal_task");
		if (task_handle == NULL) {
			err = BOARD_V4L2_ERROR_ALLOC_TASK;
			break;
		}
		board_v4l2_cxt->check_signal_task_handle = task_handle;

		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_STREAMON, &v4l2_stream_on, board_v4l2_cxt);
		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_STREAMOFF, &v4l2_stream_off, board_v4l2_cxt);
		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_BUFFER_PREPARE, &v4l2_buffer_prepare, board_v4l2_cxt);
		v4l2_model_register_callback(v4l2_handle, V4L2_MODEL_CALLBACK_BUFFER_INIT, &v4l2_buffer_init, board_v4l2_cxt);
        
        //adv7604_register_callback(board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_ADV7604], cl311mn_adv7604_event, board_v4l2_cxt);
        //gv7601_register_callback(board_v4l2_cxt->i2c_chip_handle[CL311MN_I2C_CHIP_GV7601], cl311mn_gv7601_event, board_v4l2_cxt);
        
		framegrabber_start(framegrabber_handle);

		cxt_manager_ref_context(aver_xilinx_handle);
	} while (0);

	if (err != BOARD_V4L2_OK) {
		switch (err) {
			case BOARD_V4L2_ERROR_ALLOC_TASK:
				cxt_manager_unref_context(task_model_handle);
			case BOARD_V4L2_ERROR_GET_TASK_MGR:
			case BOARD_V4L2_ERROR_GET_I2C_MGR:
			case BOARD_V4L2_ERROR_V4L2_MODEL_INIT:
			case BOARD_V4L2_ERROR_ALLOC_CXT:
			case BOARD_V4L2_ERROR_FRAMEGRABBER_INIT:
			case BOARD_V4L2_ERROR_CXT_MGR:
			default:
				break;
		}
		debug_msg("%s err %d\n", __func__, err);
	}
}

