/*
 * pic_bmp.c
 *
 *  Created on: Jun 12, 2020
 *      Author: jeff
 */

#include <linux/err.h>
#include "cxt_mgr.h"
#include "pic_bmp.h"
#include "mem_model.h"
#include "debug.h"
#include "AVerScaleVideo.h"

typedef struct
{
    BASIC_CXT_HANDLE_DECLARE;
    char *no_signal_data;
    char *out_of_range_data;
    char *copy_protection_data;

    // for scaling
    unsigned char **scaling_init_data;
    unsigned char *scaling_buffer;
}pic_bmp_context_t;

// the following data structure were moved from windows driver
#pragma pack (2)
typedef struct _BITMAPFILEHEADER {
    U16_T id;
    U32_T file_size;
    U32_T reserved;
    U32_T bitmap_data_offset;
} BITMAPFILEHEADER, *PBITMAPFILEHEADER;

typedef struct _BITMAPINFOHEADER {
    U32_T bitmap_header_size;
    U32_T width;
    U32_T height;
    U16_T planes;
    U16_T bits_per_pixel;
    U32_T compression;
    U32_T bitmap_data_size;
    U32_T h_resolution;
    U32_T v_resolution;
    U32_T used_colors;
    U32_T important_colors;
} BITMAPINFOHEADER, *PBITMAPINFOHEADER;

typedef struct _BITMAPHEADER {
    BITMAPFILEHEADER file_header;
    BITMAPINFOHEADER info_header;
    U32_T palette;              // the palette could be wrong, !!!!!
} BITMAPHEADER, *PBITMAPHEADER;

typedef struct _BITMAP {
    PBITMAPHEADER p_bitmap_header;
    char *p_bitmap_data;
} BITMAP, *PBITMAP;
#pragma pack ()

static void *pic_bmp_alloc(void);
static void pic_bmp_release(void *cxt);

static void load_bmp_file(char **rgb_data, const char *filename)
{
    void *fp = NULL;
    BITMAPHEADER header;
    int bmp_header_size = sizeof(BITMAPHEADER);
    int alloc_mem_size = 0;
    size_t read_size;

    if (!filename)
    {
        mesg("Warning: invalid bitmap file name");
        goto exit_load_bmp_file;
    }

    // open file
    fp = sys_fopen(filename, SYS_FOP_FLAG_READ);
    if (!fp)
    {
        mesg("Warning: failed to open file %s", filename);
        goto exit_load_bmp_file;
    }

    //read bmp header
    read_size = sys_fread(fp, &header, sizeof(char), bmp_header_size, 0);
    if (read_size != bmp_header_size)
    {
        mesg("Warning: failed to read file %ld//%d", read_size, bmp_header_size);
        goto exit_load_bmp_file;
    }

    // check if bmp is 640x480
    if (header.info_header.width != 640 || header.info_header.height != 480 || header.info_header.bits_per_pixel != 24)
    {
        mesg("Warning: invalid bmp format");
        goto exit_load_bmp_file;
    }

    // check file size
    alloc_mem_size = header.info_header.width * header.info_header.height * header.info_header.bits_per_pixel / 8;
    if ((header.file_header.file_size - header.file_header.bitmap_data_offset) != alloc_mem_size)
    {
#if 1 //debug log
        mesg("Warning: invalid bmp size");
        mesg("file_size:%d", header.file_header.file_size);
        mesg("bitmap size %dx%d %d bits", header.info_header.width, header.info_header.height, header.info_header.bits_per_pixel);//
        mesg("alloc_mem_size:%d", alloc_mem_size);
        mesg("bitmap_data_offset:%d", header.file_header.bitmap_data_offset);
        mesg("bitmap_header_size:%d", header.info_header.bitmap_header_size);
#endif
    }

    // allocate memory, DO NOT use kmalloc or mem_model_alloc_buffer
    *rgb_data = mem_model_new(alloc_mem_size);
    if (*rgb_data == NULL)
    {
        mesg("Error: low resource");
        goto exit_load_bmp_file;
    }
    mem_model_memset(*rgb_data, 0, alloc_mem_size);

    //read bmp raw from file
    read_size = sys_fread(fp, *rgb_data, sizeof(char), alloc_mem_size, header.file_header.bitmap_data_offset);
    if (read_size != alloc_mem_size)
    {
        mesg("Warning: failed to read file %ld//%d", read_size, alloc_mem_size);
        goto exit_load_bmp_file;
    }

    mesg("read %s successfully!!!", filename);

exit_load_bmp_file:

    if (*rgb_data == NULL)
    {
        *rgb_data = mem_model_new(640*480*3);
        if (*rgb_data)
        {
            mem_model_memset(*rgb_data, 0, 640*480*3);
            mesg("filled black instead of %s", filename);
        }
    }

    // close file
    if (fp) sys_fclose(fp);
}

static DWORD RGB_to_YUV(BYTE R, BYTE G, BYTE B)
{
    BYTE Y = 0, U = 0, V = 0;

    Y = (BYTE)(((66 * R + 129 * G + 25 * B + 128) >> 8) + 16);
    U = (BYTE)(((-38 * R - 74 * G + 112 * B + 128) >> 8) + 128);
    V = (BYTE)(((112 * R - 94 * G - 18 * B + 128) >> 8) + 128);

    return ((DWORD)(Y) << 16) + ((DWORD)(U) << 8) + V;
}

static void DumpRGBFrameToYUYV(BYTE *pRGBData, DWORD dwWidth, DWORD dwHeight, BYTE * pYUVsData)
{
    DWORD dwOffset = 0;
    DWORD dwIndex = 0;
    DWORD dwYUV_1 = 0;
    DWORD dwYUV_2 = 0;
    DWORD dwMask = 0x000000FF;
    DWORD dwIndexH, dwIndexW;

    if ((pRGBData == NULL))
    {
        return;
    }

    for (dwIndexH = 1; dwIndexH <= dwHeight; dwIndexH++)
    {
        for (dwIndexW = 0; dwIndexW < dwWidth; dwIndexW++)
        {
            dwOffset = ((dwHeight - dwIndexH)*dwWidth + dwIndexW) * 3;

            if (dwIndexW % 2 == 0)
            {
                // modify by Andy Nien, ????? 20161031, what' up ???
                //      dwYUV_1 = RGB_to_YUV(pRGBData[dwOffset + 1], pRGBData[dwOffset ], pRGBData[dwOffset +2]);
                dwYUV_1 = RGB_to_YUV(pRGBData[dwOffset + 2], pRGBData[dwOffset + 1], pRGBData[dwOffset]);
            }
            else
            {
                //dwYUV_2 = RGB_to_YUV(pRGBData[dwOffset + 1], pRGBData[dwOffset ], pRGBData[dwOffset +2]);
                dwYUV_2 = RGB_to_YUV(pRGBData[dwOffset + 2], pRGBData[dwOffset + 1], pRGBData[dwOffset]);

                // YUY2     Y1U1Y2V1
                *(pYUVsData + dwIndex) = (BYTE) ((dwYUV_1 >> 16)    &   dwMask);
                *(pYUVsData + (dwIndex + 1)) = (BYTE) ((dwYUV_1 >> 8)   &   dwMask);
                *(pYUVsData + (dwIndex + 2)) = (BYTE) ((dwYUV_2 >> 16)  &   dwMask);
                *(pYUVsData + (dwIndex + 3)) = (BYTE) (dwYUV_1                  &   dwMask);

                dwIndex += 4;
            }
        }
    }
}

static void *pic_bmp_alloc ()
{
    pic_bmp_context_t *cxt;
    cxt=mem_model_alloc_buffer(sizeof(*cxt));

    if (cxt)
    {
        // initialize
        cxt->no_signal_data = NULL;
        cxt->out_of_range_data = NULL;
        cxt->copy_protection_data = NULL;

        cxt->scaling_init_data = NULL;
        cxt->scaling_buffer = NULL;
    }

    return cxt;
}

static void pic_bmp_release(void *cxt)
{
    pic_bmp_context_t *pic_bmp_cxt=cxt;

    if(pic_bmp_cxt)
    {
        // free memory
        if (pic_bmp_cxt->no_signal_data)
        {
            mem_model_delete(pic_bmp_cxt->no_signal_data);
            pic_bmp_cxt->no_signal_data = NULL;
        }

        if (pic_bmp_cxt->out_of_range_data)
        {
            mem_model_delete(pic_bmp_cxt->out_of_range_data);
            pic_bmp_cxt->out_of_range_data = NULL;
        }

        if (pic_bmp_cxt->copy_protection_data)
        {
            mem_model_delete(pic_bmp_cxt->copy_protection_data);
            pic_bmp_cxt->copy_protection_data = NULL;
        }

        if (pic_bmp_cxt->scaling_init_data)
        {
            if (*(pic_bmp_cxt->scaling_init_data))
            {
                ScaleVideoUninit(pic_bmp_cxt->scaling_init_data);
                *(pic_bmp_cxt->scaling_init_data) = NULL;
            }

            mem_model_delete(pic_bmp_cxt->scaling_init_data);
            pic_bmp_cxt->scaling_init_data = NULL;
        }

        if (pic_bmp_cxt->scaling_buffer)
        {
            mem_model_delete(pic_bmp_cxt->scaling_buffer);
            pic_bmp_cxt->scaling_buffer = NULL;
        }

        mem_model_free_buffer(pic_bmp_cxt);
    }
}

void pic_bmp_init(cxt_mgr_handle_t cxt_mgr
        , const char *no_signal_filename
        , const char *out_of_range_filename
        , const char *copy_protection_filename)
{
    pic_bmp_context_t *pic_bmp_cxt;

    if (!cxt_mgr)
    {
        mesg("Error: invalid parameter");
        return;
    }

    pic_bmp_cxt = cxt_manager_add_cxt(cxt_mgr, PCI_BMP_CXT_ID, pic_bmp_alloc, pic_bmp_release);
    if (!pic_bmp_cxt)
    {
        mesg("Error: low resource");
        return;
    }

    pic_bmp_cxt->scaling_init_data = mem_model_new(sizeof(BYTE *));
    mem_model_memset(pic_bmp_cxt->scaling_init_data, 0, sizeof(BYTE *));
    pic_bmp_cxt->scaling_buffer = NULL;

    // allocate memory and read data from file
    load_bmp_file(&(pic_bmp_cxt->no_signal_data), no_signal_filename);
    load_bmp_file(&(pic_bmp_cxt->out_of_range_data), out_of_range_filename);
    load_bmp_file(&(pic_bmp_cxt->copy_protection_data), copy_protection_filename);
}

void pic_bmp_prepare(pic_bmp_handle_t pic_bmp_handle, unsigned long src_width, unsigned long src_height, unsigned long dst_width, unsigned long dst_height)
{
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;

    if (pic_bmp_context->scaling_buffer)
    {
        mem_model_delete(pic_bmp_context->scaling_buffer);
        pic_bmp_context->scaling_buffer = NULL;
    }
    pic_bmp_context->scaling_buffer = mem_model_new(dst_width * dst_height * 3);

    if (*(pic_bmp_context->scaling_init_data))
    {
        ScaleVideoUninit(pic_bmp_context->scaling_init_data);
        *(pic_bmp_context->scaling_init_data) = NULL;
    }
    ScaleVideoInit(pic_bmp_context->scaling_init_data, AVM_CLRSPACE_RGB24, AVM_VSCAL_BILINEAR, src_width, src_height, dst_width, dst_height);
}

static void load_image(unsigned char *dst, unsigned char *src, int dst_width, int dst_height, int src_width, int src_height, unsigned char **scaling_init_data, unsigned char *buf)
{
    if (!buf)
    {
        mesg("Error: low resource buf");
        return;
    }

    if (!scaling_init_data)
    {
        mesg("Error: low resource init_data");
        return;
    }
//    mesg("%s size:%dx%dx3=%d", __func__, width, height, width * height * 3);
    AVerDoScale(scaling_init_data, src, buf, src_width, src_height, dst_width, dst_height, AVM_CLRSPACE_RGB24, AVM_VSCAL_BILINEAR);
    DumpRGBFrameToYUYV(buf, dst_width, dst_height, dst);
}

void load_no_signal_image(pic_bmp_handle_t pic_bmp_handle, unsigned char *dst, int width, int height)
{
#if 1
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;

    if (!pic_bmp_context)
    {
        mesg("Error: invalid parameter");
        return;
    }

    if (!pic_bmp_context->no_signal_data)
    {
        mesg("Error: no_signal_data is NULL");
        return;
    }

    if (!pic_bmp_context->scaling_init_data)
    {
        mesg("Error: scaling_init_data is NULL");
        return;
    }

    if (!pic_bmp_context->scaling_buffer)
    {
        mesg("Error: scaling_buffer is NULL");
        return;
    }

    if (!dst)
    {
        mesg("Error: dst is NULL");
        return;

    }

    if ((width == 0) || (height == 0))
    {
        mesg("Error: width or height is 0");
        return;
    }

    load_image(dst, pic_bmp_context->no_signal_data, width, height, 640, 480, pic_bmp_context->scaling_init_data, pic_bmp_context->scaling_buffer);
#else
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;
    BYTE **scaling_init_data;
    unsigned char *buf = NULL;

    if (!pic_bmp_context)
    {
        mesg("Error: invalid parameter");
        return;
    }

    if (!pic_bmp_context->no_signal_data)
    {
        mesg("Error: no_signal_raw is NULL");
        return;
    }

    if (!dst)
    {
        mesg("Error: dst is NULL");
        return;

    }

    if ((width == 0) || (height == 0))
    {
        mesg("Error: width or height is 0");
        return;
    }

    buf = mem_model_new(width * height * 3);
    if (!buf)
    {
        mesg("Error: low resource buf");
        return;
    }

    scaling_init_data = mem_model_new(sizeof(BYTE *));
    if (!scaling_init_data)
    {
        mesg("Error: low resource init_data");
        mem_model_free_buffer(buf);
        return;
    }
//    mesg("%s size:%dx%dx3=%d", __func__, width, height, width * height * 3);
    AVerDoScale(scaling_init_data, pic_bmp_context->no_signal_data, buf, 640, 480, width, height, AVM_CLRSPACE_RGB24, AVM_VSCAL_BILINEAR);
    DumpRGBFrameToYUYV(buf, width, height, dst);

    mem_model_delete(scaling_init_data);
    mem_model_delete(buf);
#endif
}

void load_out_of_range_image(pic_bmp_handle_t pic_bmp_handle, unsigned char *dst, int width, int height)
{
#if 1
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;

    if (!pic_bmp_context)
    {
        mesg("Error: invalid parameter");
        return;
    }

    if (!pic_bmp_context->out_of_range_data)
    {
        mesg("Error: no_signal_raw is NULL");
        return;
    }

    if (!pic_bmp_context->scaling_init_data)
    {
        mesg("Error: scaling_init_data is NULL");
        return;
    }

    if (!pic_bmp_context->scaling_buffer)
    {
        mesg("Error: scaling_buffer is NULL");
        return;
    }

    if (!dst)
    {
        mesg("Error: dst is NULL");
        return;

    }

    if ((width == 0) || (height == 0))
    {
        mesg("Error: width or height is 0");
        return;
    }

    load_image(dst, pic_bmp_context->out_of_range_data, width, height, 640, 480, pic_bmp_context->scaling_init_data, pic_bmp_context->scaling_buffer);
#else
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;
    BYTE **scaling_init_data;
    unsigned char *buf = NULL;

    buf = mem_model_new(width * height * 3);
    if (!buf)
    {
        mesg("Error: low resource buf");
        return;
    }

    scaling_init_data = mem_model_new(sizeof(BYTE *));
    if (!scaling_init_data)
    {
        mesg("Error: low resource init_data");
        mem_model_free_buffer(buf);
        return;
    }
//    mesg("%s size:%dx%dx3=%d", __func__, width, height, width * height * 3);
    AVerDoScale(scaling_init_data, pic_bmp_context->out_of_range_data, buf, 640, 480, width, height, AVM_CLRSPACE_RGB24, AVM_VSCAL_BILINEAR);
    DumpRGBFrameToYUYV(buf, width, height, dst);

    mem_model_delete(scaling_init_data);
    mem_model_delete(buf);
#endif
}

void load_copy_protection_image(pic_bmp_handle_t pic_bmp_handle, unsigned char *dst, int width, int height)
{
#if 1
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;

    if (!pic_bmp_context)
    {
        mesg("Error: invalid parameter");
        return;
    }

    if (!pic_bmp_context->copy_protection_data)
    {
        mesg("Error: no_signal_raw is NULL");
        return;
    }

    if (!pic_bmp_context->scaling_init_data)
    {
        mesg("Error: scaling_init_data is NULL");
        return;
    }

    if (!pic_bmp_context->scaling_buffer)
    {
        mesg("Error: scaling_buffer is NULL");
        return;
    }

    if (!dst)
    {
        mesg("Error: dst is NULL");
        return;

    }

    if ((width == 0) || (height == 0))
    {
        mesg("Error: width or height is 0");
        return;
    }

    load_image(dst, pic_bmp_context->copy_protection_data, width, height, 640, 480, pic_bmp_context->scaling_init_data, pic_bmp_context->scaling_buffer);
#else
    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;
    BYTE **scaling_init_data;
    unsigned char *buf = NULL;

    buf = mem_model_new(width * height * 3);
    if (!buf)
    {
        mesg("Error: low resource buf");
        return;
    }

    scaling_init_data = mem_model_new(sizeof(BYTE *));
    if (!scaling_init_data)
    {
        mesg("Error: low resource init_data");
        mem_model_free_buffer(buf);
        return;
    }
//    mesg("%s size:%dx%dx3=%d", __func__, width, height, width * height * 3);
    AVerDoScale(scaling_init_data, pic_bmp_context->copy_protection_data, buf, 640, 480, width, height, AVM_CLRSPACE_RGB24, AVM_VSCAL_BILINEAR);
    DumpRGBFrameToYUYV(buf, width, height, dst);

    mem_model_delete(scaling_init_data);
    mem_model_delete(buf);
#endif
}

void load_black_image(pic_bmp_handle_t pic_bmp_handle, unsigned char *dst, int width, int height)
{
//    pic_bmp_context_t *pic_bmp_context = pic_bmp_handle;
//    unsigned char *buf = NULL;
//
//    if (!pic_bmp_context)
//    {
//        dst = NULL;
//        return;
//    }
//
//    buf = mem_model_alloc_buffer(width * height * 3);
//    image_scale(buf, width, height, pic_bmp_context->no_signal_raw, 640, 480);
//    rgb24_to_yuy2(dst, buf, width, height);
//
//    mem_model_free_buffer(buf);
}
